using System;

namespace wombat
{
	/// <summary>
	/// The Adventure class contains the 'world' of the game:
	/// the player and any other actors, plus the map.
	/// </summary>
	public class Adventure
	{
		private Actor _player;
		private RoomList _map = new RoomList();

		
		public Adventure()
		{
			// CONSTRUCT GAME
			//
			// FOR NOW any adventure object is always an instance of the same game
			// Later we can change this so that different games can be loaded
			//					
			/* Map is:
			 * room0--room1
			 *   | 
			 * room2--room3
			 *   |      |
			 * room4--room5
			 */

			// -- add some things to an object
			ThingList potlist = new ThingList();
			potlist.Add(new Thing("Jewel", "A lovely diamond"));
			potlist.Add(new Thing("Ring", "A ring of power."));

			// -- add some things to a room
			ThingList rm4list = new ThingList();
			rm4list.Add(new Thing("Thing one","1st thing"));
			rm4list.Add(new Thing("Thing two","2nd thing"));
			rm4list.Add(new Thing("Thing three","3rd thing"));
			rm4list.Add(new ThingHolder("a pot", "a brass container.", potlist));			

			// -- create map
			_map.Add(new Room("room0", "A field west of a white house.", dir.NOEXIT, 2, dir.NOEXIT, 1, new ThingList()));
			_map.Add(new Room("room1", "A lovely white house.", dir.NOEXIT, dir.NOEXIT, 0, dir.NOEXIT, new ThingList()));
			_map.Add(new Room("room2", "A golden room hung with rich tapestries.", 0, 4, dir.NOEXIT, 3, new ThingList()));
			_map.Add(new Room("room3", "A nasty dungeon.", dir.NOEXIT, 5, 2, dir.NOEXIT, new ThingList()));
			_map.Add(new Room("room4", "A dark cave.", 2, dir.NOEXIT, dir.NOEXIT, 5, rm4list));
			_map.Add(new Room("room5", "A sweetly scented orchard.", 3, dir.NOEXIT, 4, dir.NOEXIT, new ThingList()));
						
			// -- add player to room0
			_player = new Actor("You","The Player", _map.Item(0), new ThingList());
			
		}

		// --- Player
		public Actor Player
		{
			get
			{
				return _player;
			}
			set
			{				
				_player = value;
			}
		} // Player

		// -- Map
		public RoomList Map
		{
			get
			{
				return _map;
			}
			set
			{				
				_map = value;
			}
		} // Map

		// ========= Special Actions that control the game ==========
		//
		void moveActorTo( Actor a, Room aRoom)
		{
			a.CurrentRoom = aRoom;
		}

		int moveTo( Actor a, int direction )
		{
			Room r = a.CurrentRoom;
			int exit;

			switch( direction )
			{
				case dir.NORTH : exit = r.N;
					break;
				case dir.SOUTH : exit = r.S;
					break;
				case dir.EAST : exit = r.E;
					break;
				case dir.WEST : exit = r.W;
					break;
				default : exit = dir.NOEXIT;
					break;
			}
			if (exit != dir.NOEXIT)
				moveActorTo(a, Map.Item(exit));
			return exit;
		}

		public String movePlayerTo( int direction )
		{
			// returns string msg to display after attempting moveTo operation
			if (moveTo(_player, direction) == dir.NOEXIT)
				return msg.NOEXIT;
			else return ""; 			
		}

	} // Adventure
}
